@@ -6,10 +6,14 @@ class DriverError(base.OsbError):
     """Base Driver error"""
 
 
-class DriverConnecionError(DriverError):
+class DriverConnectionError(DriverError):
     """Driver connection error"""
 
 
+class DriverAuthenticationError(DriverError):
+    """Driver authentication error"""
+
+
 class DriverConfigError(DriverError):
     """Driver configuration error"""
 
