@@ -11,7 +11,7 @@ Configuration
 
   ---
   s3:
-    driver: s2
+    driver: s3
     aws_access_key_id: <your_ak>
     aws_secret_access_key: <your_sk>
     region: eu-west-1
@@ -19,8 +19,11 @@ Configuration
 All parameters except ``driver`` will be passed to ``boto3.resource``.
 """
 from functools import wraps
-import boto3
+
 import botocore
+import boto3
+from boto3.s3.transfer import TransferConfig
+
 from os_benchmark.drivers import base, errors
 
 
@@ -37,19 +40,24 @@ def handle_request(method):
             raise errors.DriverConnectionError(err)
         except botocore.exceptions.ClientError as err:
             code = err.response['Error']['Code']
+
+            if 'Message' not in err.response['Error']:
+                raise errors.DriverConnectionError(err)
+
             msg = err.response['Error']['Message']
             if code == '504':
                 raise errors.DriverConnectionError(err)
             if code == 'ServiceUnavailable':
                 raise errors.DriverConnectionError(err)
             if code == 'InvalidAccessKeyId':
-                msg += " (endpoint: %s)" % self.endpoint_url
+                msg += " (endpoint: %s)" % self.s3.meta.client._endpoint.host
                 raise errors.DriverAuthenticationError(msg)
             raise
     return _handle_request
 
 
 class Driver(base.RequestsMixin, base.BaseDriver):
+    id = 's3'
     default_kwargs = {}
     default_config = {}
     _default_config = {
@@ -129,14 +137,27 @@ class Driver(base.RequestsMixin, base.BaseDriver):
         return objects
 
     @handle_request
-    def upload(self, bucket_id, name, content, acl='public-read', **kwargs):
+    def upload(self, bucket_id, name, content, acl='public-read',
+               multipart_threshold=None, multipart_chunksize=None,
+               max_concurrency=None,
+               **kwargs):
         extra = {'ACL': acl}
+        multipart_threshold = multipart_threshold or 64*2**20
+        multipart_chunksize = multipart_chunksize or 64*2**20
+        max_concurrency = max_concurrency or 10
+
+        transfer_config = TransferConfig(
+            multipart_threshold=multipart_threshold,
+            max_concurrency=max_concurrency,
+            multipart_chunksize=multipart_chunksize,
+        )
         try:
             self.s3.meta.client.upload_fileobj(
-                content,
-                bucket_id,
-                name,
-                extra,
+                Fileobj=content,
+                Bucket=bucket_id,
+                Key=name,
+                ExtraArgs=extra,
+                Config=transfer_config,
             )
         except botocore.exceptions.ClientError as err:
             code = err.response['Error']['Code']
